| Safe Haskell | Safe |
|---|---|
| Language | Haskell2010 |
Data.Profunctor.Optic.Types
Contents
Synopsis
- type Optic p s t a b = p a b -> p s t
- type Optic' p s a = Optic p s s a a
- type IndexedOptic p i s t a b = p (i, a) b -> p (i, s) t
- type IndexedOptic' p i s a = IndexedOptic p i s s a a
- type CoindexedOptic p k s t a b = p a (k -> b) -> p s (k -> t)
- type CoindexedOptic' p k t b = CoindexedOptic p k t t b b
- type Iso s t a b = forall p. Profunctor p => Optic p s t a b
- type Iso' s a = Iso s s a a
- type Equality s t a b = forall p. Optic p s t a b
- type Equality' s a = Equality s s a a
- type Lens s t a b = forall p. Strong p => Optic p s t a b
- type Lens' s a = Lens s s a a
- type Ixlens i s t a b = forall p. Strong p => IndexedOptic p i s t a b
- type Ixlens' i s a = Ixlens i s s a a
- type Prism s t a b = forall p. Choice p => Optic p s t a b
- type Prism' s a = Prism s s a a
- type Cxprism k s t a b = forall p. Choice p => CoindexedOptic p k s t a b
- type Cxprism' k s a = Cxprism k s s a a
- type Grate s t a b = forall p. Closed p => Optic p s t a b
- type Grate' s a = Grate s s a a
- type Cxgrate k s t a b = forall p. Closed p => CoindexedOptic p k s t a b
- type Cxgrate' k s a = Cxgrate k s s a a
- type Affine s t a b = forall p. (Choice p, Strong p) => Optic p s t a b
- type Affine' s a = Affine s s a a
- type Ixaffine i s t a b = forall p. (Choice p, Strong p) => IndexedOptic p i s t a b
- type Ixaffine' i s a = Ixaffine i s s a a
- type Option s a = forall p. (Choice p, Strong p, forall x. Contravariant (p x)) => Optic' p s a
- type Ixoption i s a = forall p. (Choice p, Strong p, forall x. Contravariant (p x)) => IndexedOptic' p i s a
- type Grism s t a b = forall p. (Choice p, Closed p) => Optic p s t a b
- type Grism' t b = Grism t t b b
- type Traversal s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p)) => Optic p s t a b
- type Traversal' s a = Traversal s s a a
- type Ixtraversal i s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p)) => IndexedOptic p i s t a b
- type Ixtraversal' i s a = Ixtraversal i s s a a
- type Traversal1 s t a b = forall p. (Strong p, Representable p, Apply (Rep p)) => Optic p s t a b
- type Traversal1' s a = Traversal1 s s a a
- type Ixtraversal1 i s t a b = forall p. (Strong p, Representable p, Apply (Rep p)) => IndexedOptic p i s t a b
- type Ixtraversal1' i s a = Ixtraversal1 i s s a a
- type Fold s a = forall p. (Choice p, Representable p, Applicative (Rep p), forall x. Contravariant (p x)) => Optic' p s a
- type Ixfold i s a = forall p. (Choice p, Representable p, Applicative (Rep p), forall x. Contravariant (p x)) => IndexedOptic' p i s a
- type Fold1 s a = forall p. (Strong p, Representable p, Apply (Rep p), forall x. Contravariant (p x)) => Optic' p s a
- type Ixfold1 i s a = forall p. (Strong p, Representable p, Apply (Rep p), forall x. Contravariant (p x)) => IndexedOptic' p i s a
- type Cotraversal s t a b = forall p. (Choice p, Closed p, Coapplicative (Corep p), Corepresentable p) => Optic p s t a b
- type Cotraversal' t b = Cotraversal t t b b
- type PrimView s t a b = forall p. (Profunctor p, forall x. Contravariant (p x)) => Optic p s t a b
- type View s a = forall p. (Strong p, forall x. Contravariant (p x)) => Optic' p s a
- type Ixview i s a = forall p. (Strong p, forall x. Contravariant (p x)) => IndexedOptic' p i s a
- type PrimReview s t a b = forall p. (Profunctor p, Bifunctor p) => Optic p s t a b
- type Review t b = forall p. (Closed p, Bifunctor p) => Optic' p t b
- type Cxview k t b = forall p. (Closed p, Bifunctor p) => CoindexedOptic' p k t b
- type Setter s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p), Distributive (Rep p)) => Optic p s t a b
- type Setter' s a = Setter s s a a
- type Ixsetter i s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p), Distributive (Rep p)) => IndexedOptic p i s t a b
- type Ixsetter' i s a = Ixsetter i s s a a
- type Resetter s t a b = forall p. (Choice p, Closed p, Corepresentable p, Coapplicative (Corep p), Traversable (Corep p)) => Optic p s t a b
- type Resetter' s a = Resetter s s a a
- type Cxsetter k s t a b = forall p. (Choice p, Closed p, Corepresentable p, Coapplicative (Corep p), Traversable (Corep p)) => CoindexedOptic p k s t a b
- type Cxsetter' k t b = Cxsetter k t t b b
- class Branch f => Coapplicative f where
- copure :: f a -> a
- class Functor f => Branch f where
- between :: (c -> d) -> (a -> b) -> (b -> c) -> a -> d
- newtype Re p s t a b = Re {
- runRe :: p b a -> p t s
- re :: Optic (Re p a b) s t a b -> Optic p b a t s
- type (+) = Either
- type (:->) (p :: Type -> Type -> Type) (q :: Type -> Type -> Type) = forall a b. p a b -> q a b
- newtype Star (f :: Type -> Type) d c = Star {
- runStar :: d -> f c
- newtype Costar (f :: Type -> Type) d c = Costar {
- runCostar :: f d -> c
- newtype WrappedArrow (p :: Type -> Type -> Type) a b = WrapArrow {
- unwrapArrow :: p a b
- newtype Forget r a b = Forget {
- runForget :: a -> r
- class Profunctor (p :: Type -> Type -> Type) where
Optic, IndexedOptic, & CoindexedOptic
type IndexedOptic p i s t a b = p (i, a) b -> p (i, s) t Source #
type IndexedOptic' p i s a = IndexedOptic p i s s a a Source #
type CoindexedOptic p k s t a b = p a (k -> b) -> p s (k -> t) Source #
type CoindexedOptic' p k t b = CoindexedOptic p k t t b b Source #
Iso & Equality
type Iso s t a b = forall p. Profunctor p => Optic p s t a b Source #
\( \mathsf{Iso}\;S\;A = S \cong A \)
Lens
type Lens s t a b = forall p. Strong p => Optic p s t a b Source #
Lenses access one piece of a product.
\( \mathsf{Lens}\;S\;A = \exists C, S \cong C \times A \)
type Ixlens i s t a b = forall p. Strong p => IndexedOptic p i s t a b Source #
Prism
type Prism s t a b = forall p. Choice p => Optic p s t a b Source #
Prisms access one piece of a sum.
\( \mathsf{Prism}\;S\;A = \exists D, S \cong D + A \)
type Cxprism k s t a b = forall p. Choice p => CoindexedOptic p k s t a b Source #
Grate
type Grate s t a b = forall p. Closed p => Optic p s t a b Source #
Grates access the codomain of a function.
\( \mathsf{Grate}\;S\;A = \exists I, S \cong I \to A \)
type Cxgrate k s t a b = forall p. Closed p => CoindexedOptic p k s t a b Source #
Affine & Option
type Affine s t a b = forall p. (Choice p, Strong p) => Optic p s t a b Source #
A Affine processes 0 or more parts of the whole, with no interactions.
\( \mathsf{Affine}\;S\;A = \exists C, D, S \cong D + C \times A \)
type Option s a = forall p. (Choice p, Strong p, forall x. Contravariant (p x)) => Optic' p s a Source #
A Option combines at most one element, with no interactions.
type Ixoption i s a = forall p. (Choice p, Strong p, forall x. Contravariant (p x)) => IndexedOptic' p i s a Source #
Grism
Traversal, Traversal1, Fold & Fold1
type Traversal s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p)) => Optic p s t a b Source #
A Traversal processes 0 or more parts of the whole, with Applicative interactions.
\( \mathsf{Traversal}\;S\;A = \exists F : \mathsf{Traversable}, S \equiv F\,A \)
type Traversal' s a = Traversal s s a a Source #
type Ixtraversal i s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p)) => IndexedOptic p i s t a b Source #
type Ixtraversal' i s a = Ixtraversal i s s a a Source #
type Traversal1 s t a b = forall p. (Strong p, Representable p, Apply (Rep p)) => Optic p s t a b Source #
A Traversal1 processes 1 or more parts of the whole, with Apply interactions.
\( \mathsf{Traversal1}\;S\;A = \exists F : \mathsf{Traversable1}, S \equiv F\,A \)
type Traversal1' s a = Traversal1 s s a a Source #
type Ixtraversal1 i s t a b = forall p. (Strong p, Representable p, Apply (Rep p)) => IndexedOptic p i s t a b Source #
type Ixtraversal1' i s a = Ixtraversal1 i s s a a Source #
type Fold s a = forall p. (Choice p, Representable p, Applicative (Rep p), forall x. Contravariant (p x)) => Optic' p s a Source #
type Ixfold i s a = forall p. (Choice p, Representable p, Applicative (Rep p), forall x. Contravariant (p x)) => IndexedOptic' p i s a Source #
type Fold1 s a = forall p. (Strong p, Representable p, Apply (Rep p), forall x. Contravariant (p x)) => Optic' p s a Source #
type Ixfold1 i s a = forall p. (Strong p, Representable p, Apply (Rep p), forall x. Contravariant (p x)) => IndexedOptic' p i s a Source #
Cotraversal
type Cotraversal s t a b = forall p. (Choice p, Closed p, Coapplicative (Corep p), Corepresentable p) => Optic p s t a b Source #
type Cotraversal' t b = Cotraversal t t b b Source #
View & Review
type PrimView s t a b = forall p. (Profunctor p, forall x. Contravariant (p x)) => Optic p s t a b Source #
type Ixview i s a = forall p. (Strong p, forall x. Contravariant (p x)) => IndexedOptic' p i s a Source #
type PrimReview s t a b = forall p. (Profunctor p, Bifunctor p) => Optic p s t a b Source #
Setter & Resetter
type Setter s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p), Distributive (Rep p)) => Optic p s t a b Source #
A Setter modifies part of a structure.
\( \mathsf{Setter}\;S\;A = \exists F : \mathsf{Functor}, S \equiv F\,A \)
type Ixsetter i s t a b = forall p. (Choice p, Strong p, Representable p, Applicative (Rep p), Distributive (Rep p)) => IndexedOptic p i s t a b Source #
type Resetter s t a b = forall p. (Choice p, Closed p, Corepresentable p, Coapplicative (Corep p), Traversable (Corep p)) => Optic p s t a b Source #
type Cxsetter k s t a b = forall p. (Choice p, Closed p, Corepresentable p, Coapplicative (Corep p), Traversable (Corep p)) => CoindexedOptic p k s t a b Source #
Coapplicative
class Branch f => Coapplicative f where Source #
Instances
| Coapplicative Identity Source # | |
Defined in Data.Profunctor.Optic.Types | |
| Coapplicative NonEmpty Source # | |
Defined in Data.Profunctor.Optic.Types | |
| Coapplicative ((,) r) Source # | |
Defined in Data.Profunctor.Optic.Types | |
| Coapplicative (Tagged k) Source # | |
Defined in Data.Profunctor.Optic.Types | |
| Monoid m => Coapplicative ((->) m :: Type -> Type) Source # | |
Defined in Data.Profunctor.Optic.Types | |
| (Coapplicative f, Coapplicative g) => Coapplicative (Compose f g) Source # | |
Defined in Data.Profunctor.Optic.Types | |
between :: (c -> d) -> (a -> b) -> (b -> c) -> a -> d Source #
Can be used to rewrite
\g -> f . g . h
to
between f h
Re
The Re type and its instances witness the symmetry between the parameters of a Profunctor.
Instances
| (Profunctor p, forall x. Contravariant (p x)) => Bifunctor (Re p s t) Source # | |
| Cochoice p => Choice (Re p s t) Source # | |
| Choice p => Cochoice (Re p s t) Source # | |
| Costrong p => Strong (Re p s t) Source # | |
| Strong p => Costrong (Re p s t) Source # | |
| Profunctor p => Profunctor (Re p s t) Source # | |
Defined in Data.Profunctor.Optic.Types | |
| Bifunctor p => Contravariant (Re p s t a) Source # | |
type (:->) (p :: Type -> Type -> Type) (q :: Type -> Type -> Type) = forall a b. p a b -> q a b infixr 0 #
newtype Star (f :: Type -> Type) d c #
Lift a Functor into a Profunctor (forwards).
Instances
| Functor f => Representable (Star f) | |
| Applicative f => Choice (Star f) | |
| Traversable f => Cochoice (Star f) | |
| Distributive f => Closed (Star f) | |
Defined in Data.Profunctor.Closed | |
| Functor m => Strong (Star m) | |
| Functor f => Profunctor (Star f) | |
Defined in Data.Profunctor.Types | |
| Functor f => Sieve (Star f) f | |
Defined in Data.Profunctor.Sieve | |
| Monad f => Category (Star f :: Type -> Type -> Type) | |
| Monad f => Monad (Star f a) | |
| Functor f => Functor (Star f a) | |
| Applicative f => Applicative (Star f a) | |
| Distributive f => Distributive (Star f a) | |
Defined in Data.Profunctor.Types | |
| Contravariant f => Contravariant (Star f a) | |
| Alternative f => Alternative (Star f a) | |
| MonadPlus f => MonadPlus (Star f a) | |
| Apply f => Apply (Star f a) Source # | |
| type Rep (Star f) | |
Defined in Data.Profunctor.Rep | |
newtype Costar (f :: Type -> Type) d c #
Lift a Functor into a Profunctor (backwards).
Instances
| Contravariant f => Bifunctor (Costar f) Source # | |
| Functor f => Corepresentable (Costar f) | |
| Coapplicative f => Choice (Costar f) Source # | |
| Applicative f => Cochoice (Costar f) | |
| Functor f => Closed (Costar f) | |
Defined in Data.Profunctor.Closed | |
| Functor f => Costrong (Costar f) | |
| Functor f => Profunctor (Costar f) | |
Defined in Data.Profunctor.Types | |
| Functor f => Cosieve (Costar f) f | |
Defined in Data.Profunctor.Sieve | |
| Monad (Costar f a) | |
| Functor (Costar f a) | |
| Applicative (Costar f a) | |
Defined in Data.Profunctor.Types | |
| Distributive (Costar f d) | |
Defined in Data.Profunctor.Types | |
| Apply (Costar f a) Source # | |
| type Corep (Costar f) | |
Defined in Data.Profunctor.Rep | |
newtype WrappedArrow (p :: Type -> Type -> Type) a b #
Wrap an arrow for use as a Profunctor.
Constructors
| WrapArrow | |
Fields
| |
Instances
Instances
| Representable (Forget r) | |
| Monoid r => Choice (Forget r) | |
| Cochoice (Forget r) | |
| Strong (Forget r) | |
| Profunctor (Forget r) | |
Defined in Data.Profunctor.Types | |
| Sieve (Forget r) (Const r :: Type -> Type) | |
Defined in Data.Profunctor.Sieve | |
| Functor (Forget r a) | |
| Foldable (Forget r a) | |
Defined in Data.Profunctor.Types Methods fold :: Monoid m => Forget r a m -> m # foldMap :: Monoid m => (a0 -> m) -> Forget r a a0 -> m # foldr :: (a0 -> b -> b) -> b -> Forget r a a0 -> b # foldr' :: (a0 -> b -> b) -> b -> Forget r a a0 -> b # foldl :: (b -> a0 -> b) -> b -> Forget r a a0 -> b # foldl' :: (b -> a0 -> b) -> b -> Forget r a a0 -> b # foldr1 :: (a0 -> a0 -> a0) -> Forget r a a0 -> a0 # foldl1 :: (a0 -> a0 -> a0) -> Forget r a a0 -> a0 # toList :: Forget r a a0 -> [a0] # null :: Forget r a a0 -> Bool # length :: Forget r a a0 -> Int # elem :: Eq a0 => a0 -> Forget r a a0 -> Bool # maximum :: Ord a0 => Forget r a a0 -> a0 # minimum :: Ord a0 => Forget r a a0 -> a0 # | |
| Traversable (Forget r a) | |
Defined in Data.Profunctor.Types | |
| Contravariant (Forget r a) | |
| type Rep (Forget r) | |
class Profunctor (p :: Type -> Type -> Type) where #
Formally, the class Profunctor represents a profunctor
from Hask -> Hask.
Intuitively it is a bifunctor where the first argument is contravariant and the second argument is covariant.
You can define a Profunctor by either defining dimap or by defining both
lmap and rmap.
If you supply dimap, you should ensure that:
dimapidid≡id
If you supply lmap and rmap, ensure:
lmapid≡idrmapid≡id
If you supply both, you should also ensure:
dimapf g ≡lmapf.rmapg
These ensure by parametricity:
dimap(f.g) (h.i) ≡dimapg h.dimapf ilmap(f.g) ≡lmapg.lmapfrmap(f.g) ≡rmapf.rmapg
Instances
| Monad m => Profunctor (Kleisli m) | |
Defined in Data.Profunctor.Unsafe Methods dimap :: (a -> b) -> (c -> d) -> Kleisli m b c -> Kleisli m a d # lmap :: (a -> b) -> Kleisli m b c -> Kleisli m a c # rmap :: (b -> c) -> Kleisli m a b -> Kleisli m a c # (#.) :: Coercible c b => q b c -> Kleisli m a b -> Kleisli m a c # (.#) :: Coercible b a => Kleisli m b c -> q a b -> Kleisli m a c # | |
| Profunctor (Pastro p) | |
Defined in Data.Profunctor.Strong | |
| Profunctor (Environment p) | |
Defined in Data.Profunctor.Closed Methods dimap :: (a -> b) -> (c -> d) -> Environment p b c -> Environment p a d # lmap :: (a -> b) -> Environment p b c -> Environment p a c # rmap :: (b -> c) -> Environment p a b -> Environment p a c # (#.) :: Coercible c b => q b c -> Environment p a b -> Environment p a c # (.#) :: Coercible b a => Environment p b c -> q a b -> Environment p a c # | |
| Profunctor (PastroSum p) | |
Defined in Data.Profunctor.Choice Methods dimap :: (a -> b) -> (c -> d) -> PastroSum p b c -> PastroSum p a d # lmap :: (a -> b) -> PastroSum p b c -> PastroSum p a c # rmap :: (b -> c) -> PastroSum p a b -> PastroSum p a c # (#.) :: Coercible c b => q b c -> PastroSum p a b -> PastroSum p a c # (.#) :: Coercible b a => PastroSum p b c -> q a b -> PastroSum p a c # | |
| Profunctor (Coyoneda p) | |
Defined in Data.Profunctor.Yoneda Methods dimap :: (a -> b) -> (c -> d) -> Coyoneda p b c -> Coyoneda p a d # lmap :: (a -> b) -> Coyoneda p b c -> Coyoneda p a c # rmap :: (b -> c) -> Coyoneda p a b -> Coyoneda p a c # (#.) :: Coercible c b => q b c -> Coyoneda p a b -> Coyoneda p a c # (.#) :: Coercible b a => Coyoneda p b c -> q a b -> Coyoneda p a c # | |
| Profunctor (Yoneda p) | |
Defined in Data.Profunctor.Yoneda | |
| Profunctor p => Profunctor (TambaraSum p) | |
Defined in Data.Profunctor.Choice Methods dimap :: (a -> b) -> (c -> d) -> TambaraSum p b c -> TambaraSum p a d # lmap :: (a -> b) -> TambaraSum p b c -> TambaraSum p a c # rmap :: (b -> c) -> TambaraSum p a b -> TambaraSum p a c # (#.) :: Coercible c b => q b c -> TambaraSum p a b -> TambaraSum p a c # (.#) :: Coercible b a => TambaraSum p b c -> q a b -> TambaraSum p a c # | |
| Profunctor (CotambaraSum p) | |
Defined in Data.Profunctor.Choice Methods dimap :: (a -> b) -> (c -> d) -> CotambaraSum p b c -> CotambaraSum p a d # lmap :: (a -> b) -> CotambaraSum p b c -> CotambaraSum p a c # rmap :: (b -> c) -> CotambaraSum p a b -> CotambaraSum p a c # (#.) :: Coercible c b => q b c -> CotambaraSum p a b -> CotambaraSum p a c # (.#) :: Coercible b a => CotambaraSum p b c -> q a b -> CotambaraSum p a c # | |
| Profunctor (CopastroSum p) | |
Defined in Data.Profunctor.Choice Methods dimap :: (a -> b) -> (c -> d) -> CopastroSum p b c -> CopastroSum p a d # lmap :: (a -> b) -> CopastroSum p b c -> CopastroSum p a c # rmap :: (b -> c) -> CopastroSum p a b -> CopastroSum p a c # (#.) :: Coercible c b => q b c -> CopastroSum p a b -> CopastroSum p a c # (.#) :: Coercible b a => CopastroSum p b c -> q a b -> CopastroSum p a c # | |
| Profunctor p => Profunctor (Closure p) | |
Defined in Data.Profunctor.Closed Methods dimap :: (a -> b) -> (c -> d) -> Closure p b c -> Closure p a d # lmap :: (a -> b) -> Closure p b c -> Closure p a c # rmap :: (b -> c) -> Closure p a b -> Closure p a c # (#.) :: Coercible c b => q b c -> Closure p a b -> Closure p a c # (.#) :: Coercible b a => Closure p b c -> q a b -> Closure p a c # | |
| Profunctor p => Profunctor (Tambara p) | |
Defined in Data.Profunctor.Strong Methods dimap :: (a -> b) -> (c -> d) -> Tambara p b c -> Tambara p a d # lmap :: (a -> b) -> Tambara p b c -> Tambara p a c # rmap :: (b -> c) -> Tambara p a b -> Tambara p a c # (#.) :: Coercible c b => q b c -> Tambara p a b -> Tambara p a c # (.#) :: Coercible b a => Tambara p b c -> q a b -> Tambara p a c # | |
| Profunctor (Cotambara p) | |
Defined in Data.Profunctor.Strong Methods dimap :: (a -> b) -> (c -> d) -> Cotambara p b c -> Cotambara p a d # lmap :: (a -> b) -> Cotambara p b c -> Cotambara p a c # rmap :: (b -> c) -> Cotambara p a b -> Cotambara p a c # (#.) :: Coercible c b => q b c -> Cotambara p a b -> Cotambara p a c # (.#) :: Coercible b a => Cotambara p b c -> q a b -> Cotambara p a c # | |
| Profunctor (Copastro p) | |
Defined in Data.Profunctor.Strong Methods dimap :: (a -> b) -> (c -> d) -> Copastro p b c -> Copastro p a d # lmap :: (a -> b) -> Copastro p b c -> Copastro p a c # rmap :: (b -> c) -> Copastro p a b -> Copastro p a c # (#.) :: Coercible c b => q b c -> Copastro p a b -> Copastro p a c # (.#) :: Coercible b a => Copastro p b c -> q a b -> Copastro p a c # | |
| Functor f => Profunctor (Star f) | |
Defined in Data.Profunctor.Types | |
| Functor f => Profunctor (Costar f) | |
Defined in Data.Profunctor.Types | |
| Arrow p => Profunctor (WrappedArrow p) | |
Defined in Data.Profunctor.Types Methods dimap :: (a -> b) -> (c -> d) -> WrappedArrow p b c -> WrappedArrow p a d # lmap :: (a -> b) -> WrappedArrow p b c -> WrappedArrow p a c # rmap :: (b -> c) -> WrappedArrow p a b -> WrappedArrow p a c # (#.) :: Coercible c b => q b c -> WrappedArrow p a b -> WrappedArrow p a c # (.#) :: Coercible b a => WrappedArrow p b c -> q a b -> WrappedArrow p a c # | |
| Profunctor (Forget r) | |
Defined in Data.Profunctor.Types | |
| Profunctor (Tagged :: Type -> Type -> Type) | |
Defined in Data.Profunctor.Unsafe | |
| Profunctor (Conjoin j) Source # | |
Defined in Data.Profunctor.Optic.Index Methods dimap :: (a -> b) -> (c -> d) -> Conjoin j b c -> Conjoin j a d # lmap :: (a -> b) -> Conjoin j b c -> Conjoin j a c # rmap :: (b -> c) -> Conjoin j a b -> Conjoin j a c # (#.) :: Coercible c b => q b c -> Conjoin j a b -> Conjoin j a c # (.#) :: Coercible b a => Conjoin j b c -> q a b -> Conjoin j a c # | |
| Profunctor (Index a) Source # | |
Defined in Data.Profunctor.Optic.Index | |
| Profunctor (OptionRep r) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a -> b) -> (c -> d) -> OptionRep r b c -> OptionRep r a d # lmap :: (a -> b) -> OptionRep r b c -> OptionRep r a c # rmap :: (b -> c) -> OptionRep r a b -> OptionRep r a c # (#.) :: Coercible c b => q b c -> OptionRep r a b -> OptionRep r a c # (.#) :: Coercible b a => OptionRep r b c -> q a b -> OptionRep r a c # | |
| Profunctor ((->) :: Type -> Type -> Type) | |
| Functor w => Profunctor (Cokleisli w) | |
Defined in Data.Profunctor.Unsafe Methods dimap :: (a -> b) -> (c -> d) -> Cokleisli w b c -> Cokleisli w a d # lmap :: (a -> b) -> Cokleisli w b c -> Cokleisli w a c # rmap :: (b -> c) -> Cokleisli w a b -> Cokleisli w a c # (#.) :: Coercible c b => q b c -> Cokleisli w a b -> Cokleisli w a c # (.#) :: Coercible b a => Cokleisli w b c -> q a b -> Cokleisli w a c # | |
| Profunctor (GrismRep a b) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a0 -> b0) -> (c -> d) -> GrismRep a b b0 c -> GrismRep a b a0 d # lmap :: (a0 -> b0) -> GrismRep a b b0 c -> GrismRep a b a0 c # rmap :: (b0 -> c) -> GrismRep a b a0 b0 -> GrismRep a b a0 c # (#.) :: Coercible c b0 => q b0 c -> GrismRep a b a0 b0 -> GrismRep a b a0 c # (.#) :: Coercible b0 a0 => GrismRep a b b0 c -> q a0 b0 -> GrismRep a b a0 c # | |
| Profunctor (AffineRep a b) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a0 -> b0) -> (c -> d) -> AffineRep a b b0 c -> AffineRep a b a0 d # lmap :: (a0 -> b0) -> AffineRep a b b0 c -> AffineRep a b a0 c # rmap :: (b0 -> c) -> AffineRep a b a0 b0 -> AffineRep a b a0 c # (#.) :: Coercible c b0 => q b0 c -> AffineRep a b a0 b0 -> AffineRep a b a0 c # (.#) :: Coercible b0 a0 => AffineRep a b b0 c -> q a0 b0 -> AffineRep a b a0 c # | |
| Profunctor (GrateRep a b) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a0 -> b0) -> (c -> d) -> GrateRep a b b0 c -> GrateRep a b a0 d # lmap :: (a0 -> b0) -> GrateRep a b b0 c -> GrateRep a b a0 c # rmap :: (b0 -> c) -> GrateRep a b a0 b0 -> GrateRep a b a0 c # (#.) :: Coercible c b0 => q b0 c -> GrateRep a b a0 b0 -> GrateRep a b a0 c # (.#) :: Coercible b0 a0 => GrateRep a b b0 c -> q a0 b0 -> GrateRep a b a0 c # | |
| Profunctor (LensRep a b) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a0 -> b0) -> (c -> d) -> LensRep a b b0 c -> LensRep a b a0 d # lmap :: (a0 -> b0) -> LensRep a b b0 c -> LensRep a b a0 c # rmap :: (b0 -> c) -> LensRep a b a0 b0 -> LensRep a b a0 c # (#.) :: Coercible c b0 => q b0 c -> LensRep a b a0 b0 -> LensRep a b a0 c # (.#) :: Coercible b0 a0 => LensRep a b b0 c -> q a0 b0 -> LensRep a b a0 c # | |
| Profunctor (PrismRep a b) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a0 -> b0) -> (c -> d) -> PrismRep a b b0 c -> PrismRep a b a0 d # lmap :: (a0 -> b0) -> PrismRep a b b0 c -> PrismRep a b a0 c # rmap :: (b0 -> c) -> PrismRep a b a0 b0 -> PrismRep a b a0 c # (#.) :: Coercible c b0 => q b0 c -> PrismRep a b a0 b0 -> PrismRep a b a0 c # (.#) :: Coercible b0 a0 => PrismRep a b b0 c -> q a0 b0 -> PrismRep a b a0 c # | |
| Profunctor (IsoRep a b) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a0 -> b0) -> (c -> d) -> IsoRep a b b0 c -> IsoRep a b a0 d # lmap :: (a0 -> b0) -> IsoRep a b b0 c -> IsoRep a b a0 c # rmap :: (b0 -> c) -> IsoRep a b a0 b0 -> IsoRep a b a0 c # (#.) :: Coercible c b0 => q b0 c -> IsoRep a b a0 b0 -> IsoRep a b a0 c # (.#) :: Coercible b0 a0 => IsoRep a b b0 c -> q a0 b0 -> IsoRep a b a0 c # | |
| Functor f => Profunctor (Joker f :: Type -> Type -> Type) | |
Defined in Data.Profunctor.Unsafe | |
| Contravariant f => Profunctor (Clown f :: Type -> Type -> Type) | |
Defined in Data.Profunctor.Unsafe | |
| Profunctor p => Profunctor (Re p s t) Source # | |
Defined in Data.Profunctor.Optic.Types | |
| Profunctor (IxlensRep i a b) Source # | |
Defined in Data.Profunctor.Optic.Carrier Methods dimap :: (a0 -> b0) -> (c -> d) -> IxlensRep i a b b0 c -> IxlensRep i a b a0 d # lmap :: (a0 -> b0) -> IxlensRep i a b b0 c -> IxlensRep i a b a0 c # rmap :: (b0 -> c) -> IxlensRep i a b a0 b0 -> IxlensRep i a b a0 c # (#.) :: Coercible c b0 => q b0 c -> IxlensRep i a b a0 b0 -> IxlensRep i a b a0 c # (.#) :: Coercible b0 a0 => IxlensRep i a b b0 c -> q a0 b0 -> IxlensRep i a b a0 c # | |
| (Profunctor p, Profunctor q) => Profunctor (Sum p q) | |
Defined in Data.Profunctor.Unsafe | |
| (Profunctor p, Profunctor q) => Profunctor (Product p q) | |
Defined in Data.Profunctor.Unsafe Methods dimap :: (a -> b) -> (c -> d) -> Product p q b c -> Product p q a d # lmap :: (a -> b) -> Product p q b c -> Product p q a c # rmap :: (b -> c) -> Product p q a b -> Product p q a c # (#.) :: Coercible c b => q0 b c -> Product p q a b -> Product p q a c # (.#) :: Coercible b a => Product p q b c -> q0 a b -> Product p q a c # | |
| (Functor f, Profunctor p) => Profunctor (Tannen f p) | |
Defined in Data.Profunctor.Unsafe Methods dimap :: (a -> b) -> (c -> d) -> Tannen f p b c -> Tannen f p a d # lmap :: (a -> b) -> Tannen f p b c -> Tannen f p a c # rmap :: (b -> c) -> Tannen f p a b -> Tannen f p a c # (#.) :: Coercible c b => q b c -> Tannen f p a b -> Tannen f p a c # (.#) :: Coercible b a => Tannen f p b c -> q a b -> Tannen f p a c # | |
| (Profunctor p, Functor f, Functor g) => Profunctor (Biff p f g) | |
Defined in Data.Profunctor.Unsafe Methods dimap :: (a -> b) -> (c -> d) -> Biff p f g b c -> Biff p f g a d # lmap :: (a -> b) -> Biff p f g b c -> Biff p f g a c # rmap :: (b -> c) -> Biff p f g a b -> Biff p f g a c # (#.) :: Coercible c b => q b c -> Biff p f g a b -> Biff p f g a c # (.#) :: Coercible b a => Biff p f g b c -> q a b -> Biff p f g a c # | |